/*
 * applmon.h -- definitions for the char module
 *
 *********/
 
#include <linux/ioctl.h>
 
/*
 * Macros to help debugging
 */
 
#undef PDEBUG             /* undef it, just in case */
#ifdef APPLMON_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_DEBUG "applmon: " fmt, ## args)
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif
 
#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */
 
#define APPLMON_DIAG_CODE 0xdc
#define APPLMON_START_REC 0x00
#define APPLMON_STOP_REC  0x01
#define APPLMON_GEN_EVENT 0x02
#define APPLMON_START_CON 0x03
 
typedef struct ApPlist {
   short code;               /* Dianose code                   */
   char  fcn;                /* Function code                  */
   char  len;                /* Length of parameter list       */
   char  *prodid;            /* Product ID pointer             */
   short unused;             /* Reserved                       */
   short bufLen;             /* Buffer length                  */
   char  *bufAddr;           /* Buffer address                 */
} ApPlist;
 
 
typedef struct ApplMon_Dev {
   struct ApPlist plist;     /* The APPLDATA parameter list    */
   int rc;                   /* Return code from diagnose      */
   char product_id[16];      /* Product ID                     */
   char buffer[4024];        /* Monitor record buffer          */
} ApplMon_Dev;
 
#ifndef min
#  define min(a,b) ((a)<(b) ? (a) : (b))
#endif
 
/*
 * Ioctl definitions
 */
 
/* Use 'k' as magic number */
#define APPLMON_IOC_MAGIC  'k'
 
#define APPLMON_IOCRESET    _IO(APPLMON_IOC_MAGIC, 0)
 
/*
 * S means "Set" through a ptr,
 * T means "Tell" directly with the argument value
 * G means "Get": reply by setting through a pointer
 * Q means "Query": response is on the return value
 * X means "eXchange": G and S atomically
 * H means "sHift": T and Q atomically
 */
 
/* ... more to come */
 
#define APPLMON_IOCHARDRESET _IO(APPLMON_IOC_MAGIC, 1) /* debugging tool */
 
#define APPLMON_IOC_MAXNR 1
 
