/************************************************************/
/*                                                          */
/* Module ID  - main.c                                      */
/*                                                          */
/* Function   - Provide interface between Linux/390 (VM) &  */
/*              various CP interfaces.                      */
/*                                                          */
/* Called By  -                                             */
/*                                                          */
/* Calling To -                                             */
/*                                                          */
/* Parameters - (1)                                         */
/*              DESCRIPTION:                                */
/*                                                          */
/* Notes      - (1) ....................................... */
/*                                                          */
/*              (2) ....................................... */
/*                                                          */
/*                                                          */
/* Name       - Neale Ferguson.                             */
/*                                                          */
/* Date       - January, 2000.                              */
/*                                                          */
/*                                                          */
/* Associated    - (1) Refer To ........................... */
/* Documentation                                            */
/*                 (2) Refer To ........................... */
/*                                                          */
/************************************************************/
 
/************************************************************/
/*                                                          */
/*                     DEFINES                              */
/*                     -------                              */
/*                                                          */
/************************************************************/
 
#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif
 
#define __NO_VERSION__ /* don't define kernel_verion in module.h */
 
/*=============== End of Defines ===========================*/
 
/************************************************************/
/*                                                          */
/*              INCLUDE STATEMENTS                          */
/*              ------------------                          */
/*                                                          */
/************************************************************/
 
#include <linux/version.h>
#include <linux/module.h>
 
#include <linux/kernel.h> /* printk() */
#include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>        /* O_ACCMODE */
 
#include <asm/system.h>   /* cli(), *_flags */
#include <asm/segment.h>  /* memcpy and such */
#include <asm/ebcdic.h>   /* ebcdic stuff */
#include <asm/uaccess.h>  /* copy to/from user space */
 
#include "cpint.h"        /* local definitions */
 
/*================== End of Include Statements =============*/
 
/************************************************************/
/*                                                          */
/*              TYPE DEFINITIONS                            */
/*              ----------------                            */
/*                                                          */
/************************************************************/
 
 
/*================== End of Type Definitions ===============*/
 
/************************************************************/
/*                                                          */
/*             FUNCTION PROTOTYPES                          */
/*             -------------------                          */
/*                                                          */
/************************************************************/
 
int cpint_open (struct inode *, struct file *);
int init_module(void);
void cleanup_module(void);
 
/*================== End of Prototypes =====================*/
 
/************************************************************/
/*                                                          */
/*           GLOBAL VARIABLE DECLARATIONS                   */
/*           ----------------------------                   */
/*                                                          */
/************************************************************/
 
char kernel_version [] = UTS_RELEASE;
 
int cpint_major   = CPINT_MAJOR;
int cpint_nr_devs = CPINT_NR_DEVS;
 
CPInt_Dev *cpint_devices;
 
/*----------------------------------------------------------*/
/* The different file operations                            */
/*----------------------------------------------------------*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
struct file_operations cpint_fops = {
    open:	cpint_open,
};
 
struct file_operations cpcmd_fops = {
    read:  	cpcmd_read,
    write: 	cpcmd_write,
    open: 	cpcmd_open,
    ioctl:	cpcmd_ioctl,
    release:	cpcmd_release,
};
 
struct file_operations applmon_fops = {
    write:	applmon_write,
    open:	applmon_open,
    release:	applmon_release,
};
#else
struct file_operations cpint_fops = {
    NULL,          /* cpint_lseek                           */
    NULL,          /* cpint_read                            */
    NULL,          /* cpint_write                           */
    NULL,          /* cpint_readdir                         */
    NULL,          /* cpint_select                          */
    NULL,
    NULL,          /* cpint_mmap                            */
    cpint_open,
    NULL,          /* cpint_flush                           */
    NULL,          /* cpint_release                         */
                   /* nothing more, fill with NULLs         */
};

struct file_operations cpcmd_fops = {
    NULL,          /* cpcmd_lseek                           */
    cpcmd_read,
    cpcmd_write,
    NULL,          /* cpcmd_readdir                         */
    NULL,          /* cpcmd_select                          */
    NULL,
    NULL,          /* cpcmd_mmap                            */
    cpcmd_open,
    NULL,          /* cpcmd_flush                           */
    cpcmd_release,
                   /* nothing more, fill with NULLs         */
};

struct file_operations applmon_fops = {
    NULL,          /* applmon_lseek                         */
    NULL,
    applmon_write,
    NULL,          /* applmon_readdir                       */
    NULL,          /* applmon_select                        */
    NULL,
    NULL,          /* applmon_mmap                          */
    applmon_open,
    NULL,          /* applmon_flush                         */
    applmon_release,
                   /* nothing more, fill with NULLs         */
};
#endif
 
/*----------------------------------------------------------*/
/* Map the minor number to an index into a file-ops table   */
/*----------------------------------------------------------*/
static int fopMap[256] =
{-1, -1, -1, -1, -1, -1, -1, -1,  0, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 , 1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 ,-1, -1, -1, -1};
 
/*----------------------------------------------------------*/
/* Array of file-operations structures for each device type */
/*----------------------------------------------------------*/
struct file_operations *cpFops[] = {
   &cpcmd_fops,
   &applmon_fops
};
 
/*============== End of Variable Declarations ==============*/
 
/************************************************************/
/*                                                          */
/* Name       - cpint_open.                                 */
/*                                                          */
/* Function   - Determine what CP interface "device" the    */
/*              caller requires. Check that it exists and   */
/*              if so, install the appropriate file_ops     */
/*              pointers and call the required open routine.*/
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*                                                          */
/************************************************************/
 
int
cpint_open (struct inode *inode, struct file *filp)
{
 int num  = MINOR(inode->i_rdev),
     opIndex;
 CPInt_Dev *dev;
 
    if (num >= cpint_nr_devs)
       return -ENODEV;
 
    dev = &cpint_devices[num];
 
    opIndex = fopMap[num];
    if (opIndex > -1) {
       filp->f_op = cpFops[opIndex];
       return(filp->f_op->open(inode, filp));
    }
 
    return -ENODEV;
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - init_module.                                */
/*                                                          */
/* Function   - Initialize this device driver.              */
/*                                                          */
/************************************************************/
 
int
init_module(void)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
 int result,
     i_dev;
 CPInt_Dev *dev;
#else
 int result;
#endif
 
    /*------------------------------------------------------*/
    /* Register your major, and accept a dynamic number     */
    /*------------------------------------------------------*/
    result = register_chrdev(cpint_major, "cpint", &cpint_fops);
    if (result < 0) {
        printk(KERN_WARNING "cpint: can't get major %d\n",cpint_major);
        return result;
    }
    if (cpint_major == 0)
       cpint_major = result;
 
    /*------------------------------------------------------*/
    /* allocate the devices -- we can't have them static,   */
    /* as the number can be specified at load time          */
    /*------------------------------------------------------*/
    cpint_devices = kmalloc(cpint_nr_devs * sizeof(CPInt_Dev), GFP_KERNEL);
    if (!cpint_devices) {
        result = -ENOMEM;
        unregister_chrdev(cpint_major, "cpint");
        return result;
    }
 
    memset(cpint_devices, 0, cpint_nr_devs * sizeof (CPInt_Dev));
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
    for (dev = cpint_devices, i_dev = 0; 
         i_dev < cpint_nr_devs; 
         dev++, i_dev++)
       init_waitqueue_head(&dev->devWait);
#endif
 
    return 0;
 
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - cleanup_module.                             */
/*                                                          */
/* Function   - Unload this device driver.                  */
/*                                                          */
/************************************************************/
 
void
cleanup_module(void)
{
 int i_dev;
 
    unregister_chrdev(cpint_major, "cpint");
    for (i_dev = 0; i_dev < CPINT_NR_DEVS; i_dev++) {
        if (cpint_devices[i_dev].devExt)
           kfree(cpint_devices[i_dev].devExt);
    }
    kfree(cpint_devices);
 
}
 
/*===================== End of Function ====================*/
