/************************************************************/
/*                                                          */
/* Module ID  - monmain.c                                   */
/*                                                          */
/* Function   - Interface to the CP APPLMON system service. */
/*                                                          */
/* Called By  - N/A.                                        */
/*                                                          */
/* Calling To - N/A.                                        */
/*                                                          */
/* Parameters - See individual entry points.                */
/*                                                          */
/* Notes      - (1) ....................................... */
/*                                                          */
/*              (2) ....................................... */
/*                                                          */
/*                                                          */
/* Name       - Neale Ferguson.                             */
/*                                                          */
/* Date       - January, 2000.                              */
/*                                                          */
/*                                                          */
/* Associated    - (1) Refer To ........................... */
/* Documentation                                            */
/*                 (2) Refer To ........................... */
/*                                                          */
/************************************************************/
 
/************************************************************/
/*                                                          */
/*                     DEFINES                              */
/*                     -------                              */
/*                                                          */
/************************************************************/
 
#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif
 
#define __NO_VERSION__ /* don't define kernel_verion in module.h */
 
#ifdef __s390x__
# define KM_FLAGS	GFP_ATOMIC|GFP_DMA
#else
# define KM_FLAGS	GFP_ATOMIC
#endif

/*=============== End of Defines ===========================*/
 
/************************************************************/
/*                                                          */
/*              INCLUDE STATEMENTS                          */
/*              ------------------                          */
/*                                                          */
/************************************************************/
 
#include <linux/module.h>
#include <linux/version.h>
 
#include <linux/kernel.h> /* printk() */
#include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>        /* O_ACCMODE */
 
#include <asm/system.h>   /* cli(), *_flags */
#include <asm/segment.h>  /* memcpy and such */
#include <asm/ebcdic.h>   /* ebcdic stuff */
#include <asm/uaccess.h>  /* copy to/from user space */
 
#include "cpint.h"        /* local definitions */
#include "applmon.h"      /* specific definitions */
 
/*================== End of Include Statements =============*/
 
/************************************************************/
/*                                                          */
/*              TYPE DEFINITIONS                            */
/*              ----------------                            */
/*                                                          */
/************************************************************/
 
/*================== End of Type Definitions ===============*/
 
/************************************************************/
/*                                                          */
/*             FUNCTION PROTOTYPES                          */
/*             -------------------                          */
/*                                                          */
/************************************************************/
 
static int inline applMonGen(signed char, ApplMon_Dev *);
 
/*================== End of Prototypes =====================*/
 
/************************************************************/
/*                                                          */
/*           GLOBAL VARIABLE DECLARATIONS                   */
/*           ----------------------------                   */
/*                                                          */
/************************************************************/
 
ApplMon_Dev *applmon_devices;
 
/*============== End of Variable Declarations ==============*/
 
/************************************************************/
/*                                                          */
/* Name       - applMonGen.                                 */
/*                                                          */
/* Function   - Issues DIAG DC to talk to monitor service.  */
/*                                                          */
/* Parameters - fcn    - Function code.                     */
/*              devExt - Device extension.                  */
/*                                                          */
/************************************************************/
 
static int inline
applMonGen(signed char fcn, ApplMon_Dev *devExt)
{
 int rc;
 ApPlist *plist;
 
   if ((fcn < APPLMON_START_REC) ||
       (fcn > APPLMON_START_CON))
      return -EINVAL;
 
   plist = &devExt->plist;
 
   plist->fcn = fcn;
 
#ifdef __s390x__
   asm volatile ("LRA   2,0(%1)\n\t"
		 "SAM31\n\t"
                 ".long 0x832300dc\n\t"
		 "SAM64\n\t"
                 "LGFR  %0,3\n\t"
                 : "=d" (rc)
                 : "a" (plist)
                 : "2", "3");
#else
   asm volatile ("LRA   2,0(%1)\n\t"
                 ".long 0x832300dc\n\t"
                 "LR    %0,3\n\t"
                 : "=d" (rc)
                 : "a" (plist)
                 : "2", "3");
#endif
 
   if (rc != 0)
      rc = -EINVAL;
 
   return(rc);
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - applmon_open.                               */
/*                                                          */
/* Function   - Open the CP command device.                 */
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*                                                          */
/************************************************************/
 
int
applmon_open (struct inode *inode, struct file *filp)
{
 int num  = MINOR(inode->i_rdev),
     rc;
 CPInt_Dev   *dev;
 ApplMon_Dev *devExt;
 
    if (num >= cpint_nr_devs)
       return -ENODEV;
 
    dev = &cpint_devices[num];
 
    if (!dev->devExt)
       dev->devExt = kmalloc(sizeof(ApplMon_Dev), KM_FLAGS);
 
    if (!dev->devExt)
       return -ENOMEM;
 
    devExt = dev->devExt;
    memset(devExt, 0, sizeof(ApplMon_Dev));
 
    /*-----------------------------------------------------*/
    /* use filp->private_data to point to the device data  */
    /*-----------------------------------------------------*/
    filp->private_data = dev;
 
    if (dev->devLock == 0) {
       memset(devExt->product_id, 0, sizeof(devExt->product_id));
       devExt->plist.code    = APPLMON_DIAG_CODE;
       devExt->plist.len     = sizeof(devExt->plist);
       devExt->plist.prodid  = &devExt->product_id[0];
       devExt->plist.unused  = 0;
       devExt->plist.bufLen  = sizeof(devExt->buffer);
       devExt->plist.bufAddr = (char *) &devExt->buffer;
       if ((rc = applMonGen(APPLMON_START_REC, devExt)) != 0)
          return(rc);
    }
 
    dev->devLock++;
    MOD_INC_USE_COUNT;
    return 0;
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - applmon_release.                            */
/*                                                          */
/* Function   - Close a device.                             */
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*                                                          */
/************************************************************/
 
int
applmon_release (struct inode *inode, struct file *filp)
{
 CPInt_Dev   *dev = filp->private_data;
 ApplMon_Dev *devExt;
 
   devExt = dev->devExt;
   dev->devLock--;
   if (dev->devLock == 0)
      applMonGen(APPLMON_STOP_REC, devExt);
 
   MOD_DEC_USE_COUNT;
 
   return(0);
}
 
/*===================== End of Function ====================*/
 
/************************************************************/
/*                                                          */
/* Name       - applmon_write.                              */
/*                                                          */
/* Function   - Execute a CP command and place result into  */
/*              a response buffer.                          */
/*                                                          */
/* Parameters - inode -                                     */
/*              filp  -                                     */
/*              buf   - Pointer to read buffer.             */
/*              count - Size of read buffer.                */
/*                                                          */
/************************************************************/
 
ssize_t
applmon_write (struct file *filp, const char *buf,
               count_t count, loff_t *ppos)
{
 CPInt_Dev   *dev    = filp->private_data;
 ApplMon_Dev *devExt = dev->devExt;
 int rc;
 char monLen, fcn;
 struct monData {
   char fcn;
   char len;
   char data[4204];
 } *monData;
 
    if (count <= 4)
       return -EINVAL;
 
    monData = (void *) buf;
 
    copy_from_user(&monLen, &monData->len, sizeof(monLen));
 
    if (monLen > sizeof(devExt->buffer))
       return -EINVAL;
 
    copy_from_user(&fcn, &monData->fcn, sizeof(fcn));
    copy_from_user(devExt->buffer, monData->data, monLen);
 
    if ((fcn <= APPLMON_STOP_REC) &&
        (fcn > APPLMON_START_CON))
       return -EINVAL;
 
    if ((rc = applMonGen(fcn, devExt)) != 0)
       return(rc);
 
    filp->f_pos = 0;
    return count;
}
 
/*===================== End of Function ====================*/
